# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from ..nx import AverageNetworks


def test_AverageNetworks_inputs():
    input_map = dict(
        group_id=dict(
            usedefault=True,
        ),
        in_files=dict(
            mandatory=True,
        ),
        out_gexf_groupavg=dict(
            extensions=None,
        ),
        out_gpickled_groupavg=dict(
            extensions=None,
        ),
        resolution_network_file=dict(
            extensions=None,
        ),
    )
    inputs = AverageNetworks.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value


def test_AverageNetworks_outputs():
    output_map = dict(
        gexf_groupavg=dict(
            extensions=None,
        ),
        gpickled_groupavg=dict(
            extensions=None,
        ),
        matlab_groupavgs=dict(),
    )
    outputs = AverageNetworks.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
